package cn.com.ty.lift.business.annualinspection.service.impl;

import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.mapper.AnnualInspectionMapper;
import cn.com.ty.lift.business.annualinspection.service.IAnnualInspectionService;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.LiftHistory;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.service.LiftHistoryService;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.library.service.PlatformCompanyLiftRelevanceService;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.utils.Judge;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * <p>
 * 年检记录 服务实现类
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Service
@AllArgsConstructor
public class AnnualInspectionService extends ServiceImpl<AnnualInspectionMapper, AnnualInspection> implements IAnnualInspectionService {

    private MaintenancePlanService              maintenancePlanService;
    private LiftService                         liftService;
    private LiftHistoryService                  liftHistoryService;
    private PlatformCompanyLiftRelevanceService platformCompanyLiftRelevanceService;

    /**
     * 年检列表中根据条件分页查询记录
     * @param request InspectionRequest
     * @return IPage<InspectionResponse>
     */
    public IPage<InspectionResponse> pageByCondition(InspectionRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<InspectionResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByCondition(page, request);
    }

    /**
     * 年检列表中根据条件计算记录总条数
     * @param request InspectionRequest
     * @return long
     */
    public long countByCondition(InspectionRequest request) {
        return baseMapper.countByCondition(request);
    }

    /**
     * 根据id查询单个年检记录的详情，带全部关联信息
     * @param request InspectionRequest
     * @return InspectionResponse
     */
    public InspectionResponse selectByIdWithInfo(InspectionRequest request) {
        return baseMapper.selectByIdWithInfo(request);
    }

    /**
     * 根据用户id查询关联的年检
     * @param request InspectionRequest
     * @return IPage<InspectionResponse>
     */
    public IPage<InspectionResponse> pageByUser(InspectionRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<InspectionResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByUser(page, request);
    }

    /**
     * 阶段2 修改电梯的业务状态为年检中
     * 阶段3 如果合格，修改电梯的业务状态为正常，
     * 完成年检，如果是不合格，清空维保计划，如果是合格，直接更新
     * @param entity AnnualInspection
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean modify(AnnualInspection entity) {
        //更新年检
        boolean up = saveOrUpdate(entity);
        //更新失败，直接返回false
        if (!up) {
            return false;
        }

        Integer stepStatus = entity.getStepStatus();
        Integer stepName = entity.getStepName();
        Long mtCompanyId = entity.getMtCompanyId();
        Long liftId = entity.getLiftId();

        //阶段2 修改电梯的业务状态为年检中
        if (Judge.Inspection.stepTwo(stepName)) {
            PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId, liftId);
            if (null == relevance) {
                return false;
            }
            relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.ANNUAL_INSPECTION.getCode());
            boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
            if (re) {
                return true;
            } else {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }

        //阶段3 如果合格，修改电梯的业务状态为正常，
        if (Judge.Inspection.stepThree(stepName)) {
            PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId, liftId);
            if (null == relevance) {
                return false;
            }
            relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.NORMAL.getCode());
            boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
            if (re) {
                return true;
            } else {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }

        //阶段4 年检结果不合格 or 合格
        if (Judge.Inspection.stepFour(stepName)) {
            //todo 不合格：流程结束，电梯扭转到不合格状态列表，电梯停保，清理维保计划记录
            if (Judge.Inspection.statusCertificateReportUnqualified(stepStatus)) {
                boolean mp = maintenancePlanService.cleanPlan(mtCompanyId, liftId);
                if (mp) {
                    return true;
                } else {
                    //强制手动事务回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return false;
                }
            }
            //todo:合格：修改电梯的年检时间lift-annual_inspection_date，增加lift_history记录
            if (Judge.Inspection.statusCertificateReportQualified(stepStatus)) {
                //电梯不存在，说明数据有误，直接返回false
                Lift lift = liftService.getById(liftId);
                if (null == lift) {
                    return false;
                }
                //更新电梯之前获取旧的年检时间
                LocalDate oldInspectionDate = lift.getAnnualInspectionDate();
                LocalDate nextInspectionTime = entity.getNextInspectionTime();
                lift.setAnnualInspectionDate(nextInspectionTime);
                boolean li = liftService.updateById(lift);
                //更新电梯失败，返回false
                if (!li) {
                    return false;
                }
                //更新电梯成功，插入电梯操作记录
                LiftHistory liftHistory = new LiftHistory();
                liftHistory.setId(IdWorker.getId());
                liftHistory.setMtCompanyId(mtCompanyId);
                liftHistory.setLiftId(liftId);
                liftHistory.setCode(lift.getLiftCode());
                liftHistory.setDescription("修改电梯年检日期");
                liftHistory.setContent(StrUtil.format("电梯年检日期由:{}修改成{}", oldInspectionDate, nextInspectionTime));
                //fixme:获取当前登录用户
                liftHistory.setOperatorId(entity.getCreatorId());
                liftHistory.setOperateDate(LocalDateTime.now());
                boolean lh = liftHistoryService.save(liftHistory);
                //插入电梯记录成功，直接返回true,否则手动事务回滚，返回false
                if (lh) {
                    return true;
                } else {
                    //强制手动事务回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return false;
                }
            }
        }
        return true;
    }
}
