package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.LiftBrand;
import cn.com.ty.lift.common.constants.RedisConstants;
import cn.com.ty.lift.common.model.AreaCode;
import cn.com.ty.lift.common.utils.AliyunOSS;
import cn.com.ty.lift.common.utils.Judge;
import cn.com.ty.lift.common.utils.OssEntity;
import cn.com.ty.lift.common.utils.StrPool;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author bieao
 * @date 2019/12/19
 * @description 公共初始化数据
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("common")
public class CommonController {

    private static final String DateInPath = "yyyy/MM/dd";

    private RedisTemplate<String, Object> redisTemplate;
    private SystemConfiguration           systemConfiguration;
    private AliyunOSS                     aliyunOSS;

    /**
     * @return RestResponse 省、市、区树形结构列表
     * @description 查询省、市、区树形结构列表
     * @date 2019/12/4 10:23 AM
     */
    @PostMapping("area/list")
    @SuppressWarnings("unchecked")
    public RestResponse areaList() {
        List<AreaCode> areaCodeList = (List<AreaCode>)redisTemplate.opsForValue().get(RedisConstants.RK_AREA_LIST);
        if (ObjectUtil.isEmpty(areaCodeList)) {
            return RestResponse.success();
        }
        return RestResponse.success(areaCodeList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @return RestResponse 电梯品牌列表
     * @description 查询电梯品牌列表
     * @date 2019/12/10 10:23 AM
     */
    @PostMapping("brand/list")
    @SuppressWarnings(value = "unchecked")
    public RestResponse list(){
        List<LiftBrand> liftBrandList = (List<LiftBrand>) redisTemplate.opsForValue().get(RedisConstants.RK_LIFT_BRAND_LIST);
        if (ObjectUtil.isEmpty(liftBrandList)) {
            return RestResponse.success();
        }
        return RestResponse.success(liftBrandList, MessageUtils.get("msg.query.success"));
    }

    /**
     * 批量上传多个文件
     * @param files 文件列表
     * @return RestResponse
     */
    @PostMapping("uploads")
    public RestResponse uploads(@RequestParam("files") MultipartFile[] files){
        try {
            Judge.notNull(files,Judge.Upload.fileDataMissing);
            List<OssEntity> ossEntities = new ArrayList<>();
            for (MultipartFile file : files) {
                String fileName = handleFile(file);
                if(StrUtil.isEmpty(fileName)){
                    return RestResponse.fail(Judge.Upload.fileFormatNotSupport);
                }
                OssEntity entity = new OssEntity();
                entity.setObjectName(fileName);
                entity.setContent(file.getBytes());
                ossEntities.add(entity);
            }
            Judge.notNull(ossEntities);
            Map<String, String> urls = aliyunOSS.putObjects(systemConfiguration.getBucketName(), ossEntities);
            Judge.notNull(urls);
            return RestResponse.success(urls.values());
        }catch (Exception e){
            log.error("上传文件异常: {}",e);
            return RestResponse.fail(Judge.Upload.fileUploadFail);
        }
    }

    /**
     * 上传单个文件
     * @param file 上传的文件
     * @return RestResponse
     */
    @PostMapping("upload")
    public RestResponse upload(@RequestParam("file") MultipartFile file){
        try {
            String fileName = handleFile(file);
            if(StrUtil.isEmpty(fileName)){
                return RestResponse.fail(Judge.Upload.fileFormatNotSupport);
            }
            String url = aliyunOSS.putObject(systemConfiguration.getBucketName(), fileName, file.getBytes());
            log.info("上传文件，文件URL: {}",url);
            return RestResponse.success(url);
        }catch (Exception e){
            log.error("上传文件异常: {}",e);
            return RestResponse.fail(Judge.Upload.fileUploadFail);
        }
    }

    /**
     * 校验文件大小，文件格式，生成新文件名
     * @param file
     * @return
     */
    private String handleFile(MultipartFile file){
        Judge.notTrue(null == file || file.isEmpty(),Judge.Upload.fileDataMissing);
        // 获取文件名，带后缀
        String originalFilename = file.getOriginalFilename();
        log.info("上传文件，原文件名：{}", originalFilename);
        // 获取文件的后缀格式
        String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf(StrPool.DOT)).toLowerCase();
        Judge.notNull(fileSuffix,Judge.Upload.fileFormatIllegal);
        if(StrUtil.equalsAny(fileSuffix,Judge.Upload.pics)){
            Judge.notTrue(file.getSize() > Judge.Upload.maxSizePic,"文件大小不超过" + Judge.Upload.maxSizePicDesc);
        }else if(StrUtil.equalsAny(fileSuffix,Judge.Upload.files)){
            Judge.notTrue(file.getSize() > Judge.Upload.maxSizeFile,"文件大小不超过" + Judge.Upload.maxSizeFileDesc);
        }else if(StrUtil.equalsAny(fileSuffix,Judge.Upload.videos)){
            Judge.notTrue(file.getSize() > Judge.Upload.maxSizeVideo,"文件大小不超过" + Judge.Upload.maxSizeVideoDesc);
        }else{
            return null;
        }
        String fileName = StrUtil.format("{}/{}{}",DateUtil.format(DateUtil.date(),DateInPath),IdWorker.getIdStr(),fileSuffix);
        log.info("上传文件，新文件名：{}", fileName);
        return fileName;
    }
}
