package cn.com.ty.lift.business.contract.service;

import cn.com.ty.lift.business.contract.dao.entity.Contracts;
import cn.com.ty.lift.business.contract.dao.entity.ContractsExtend;
import cn.com.ty.lift.business.contract.dao.entity.ContractsHistory;
import cn.com.ty.lift.business.contract.dao.entity.Payment;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractResponse;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractsHistoryRequest;
import cn.com.ty.lift.business.contract.dao.mapper.ContractsMapper;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author bieao
 * @date 2019/12/07
 * @description 合同管理业务层
 */
@Service
@Slf4j
public class ContractService extends ServiceImpl<ContractsMapper, Contracts> {

    @Resource
    private ContractsMapper contractsMapper;

    @Resource
    private ContractHistoryService contractHistoryService;

    @Resource
    private PaymentService paymentService;

    @Resource
    private ProjectService projectService;

    /**
     * @param request 合同列表查询条件
     * @return RestResponse 合同分页列表结果
     * @description 查询合同列表
     * @date 2019/12/7 11:53 AM
     */
    public IPage<ContractResponse> list(ContractRequest request) {
        IPage<ContractResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        if (ObjectUtil.isNotEmpty(request.getContractId())) {
            request.setContractId(StrUtil.format("%{}%", request.getContractId()));
        }
        if (ObjectUtil.isNotEmpty(request.getProjectName())) {
            request.setProjectName(StrUtil.format("%{}%", request.getProjectName()));
        }
        return contractsMapper.findByCondition(page, request);
    }

    /**
     * @param request 合同编号
     * @return RestResponse 合同历史记录分页列表结果
     * @description 查询合同历史记录列表
     * @date 2019/12/7 11:53 AM
     */
    public IPage<ContractsHistory> list(ContractsHistoryRequest request) {
        return contractHistoryService.list(request);
    }

    /**
     * @param contracts 合同信息
     * @param extend 项目名称、项目地址、台量
     * @return 是否成功
     * @description 保存项目信息
     * @date 2020/1/14 4:30 下午
     */
    public Project saveProject(Contracts contracts, ContractsExtend extend, Project project) {
        project.setProjectName(extend.getProjectName());
        project.setAddress(extend.getProjectAddress());
        project.setNum(contracts.getLiftNum());
        project.setStartDate(contracts.getStarDate());
        project.setEndDate(contracts.getEndDate());
        project.setMtCompanyId(contracts.getMtCompanyId());
        return projectService.saveProject(project);
    }

    /**
     * @param extend 新增合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增合同
     * @date 2019/12/7 3:31 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(ContractsExtend extend) {
        Project project = new Project();
        project.setId(IdWorker.getId());
        Contracts contracts = extend.getContracts();
        if (ObjectUtil.isEmpty(contracts.getCode())) {
            //按当前日期时间戳自动生成合同编号
            String contractId = DateUtils.generateCode();
            contracts.setCode(contractId);
        }
        contracts.setProjectId(project.getId());
        //插入合同信息
        int result = contractsMapper.insert(contracts);
        if (result < 0) {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //插入项目信息
        Project info = saveProject(contracts, extend, project);
        if (ObjectUtil.isEmpty(info)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //批量插入收款信息
        boolean ret = paymentService.insertBatch(extend.getPaymentList(), contracts.getId());
        if (!ret) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.batch.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param extend 修改合同、项目、付款信息数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 合同变更
     * @date 2019/12/7 3:31 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse modify(ContractsExtend extend) {
        Contracts contracts = extend.getContracts();
        //修改合同信息
        boolean contractResult = updateById(contracts);
        if (!contractResult) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        //修改付款信息
        boolean paymentResult = paymentService.modifyBatchById(extend.getPaymentList());
        if (!paymentResult) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param id 主键id
     * @return Contracts 合同详情
     * @description 查看合同详情
     * @date 2019/12/7 11:50 AM
     */
    public Map<String, Object> detail(Long id) {
        QueryWrapper<Contracts> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        Contracts contracts = getOne(queryWrapper);
        List<Payment> paymentMap = paymentService.paymentList(id);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("contract", contracts);
        resultMap.put("paymentList", paymentMap);
        Project project = projectService.getOne(contracts.getProjectId());
        resultMap.put("project", project);
        return resultMap;
    }
}
