package cn.com.ty.lift.business.emergency.service.impl;

import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.entity.ErRecordCost;
import cn.com.ty.lift.business.emergency.entity.ErRecordImg;
import cn.com.ty.lift.business.emergency.mapper.EmergencyRepairMapper;
import cn.com.ty.lift.business.emergency.service.IEmergencyRepairService;
import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.service.PlatformCompanyLiftRelevanceService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.utils.Judge;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 临修记录 服务实现类
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Service
@AllArgsConstructor
public class EmergencyRepairService extends ServiceImpl<EmergencyRepairMapper, EmergencyRepair> implements IEmergencyRepairService {

    private PlatformCompanyLiftRelevanceService platformCompanyLiftRelevanceService;
    private ErRecordImgService                  erRecordImgService;
    private ErRecordCostService                 erRecordCostService;
    private EvaluationService                   evaluationService;

    /**
     * 分页查询急修中的急修记录
     *
     * @param request RepairRequest
     * @return IPage<RepairResponse>
     */
    public IPage<RepairResponse> pageByRepairing(RepairRequest request) {
        long pageSize = request.getPageSize();
        long pageNum = request.getPageNum();
        IPage<RepairResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByRepairing(page, request);
    }

    /**
     * 根据条件分页查询急修记录
     *
     * @param request RepairRequest
     * @return IPage<RepairResponse>
     */
    public IPage<RepairResponse> pageByCondition(RepairRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<RepairResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByCondition(page, request);
    }

    /**
     * 根据条件统计急修记录总条数
     *
     * @param request RepairRequest
     * @return long
     */
    public long countByCondition(RepairRequest request) {
        return baseMapper.countByCondition(request);
    }

    /**
     * 根据id查询急修记录，带关联信息
     *
     * @param request
     * @return RepairResponse
     */
    public RepairResponse selectByIdWithInfo(RepairRequest request) {
        return baseMapper.selectByIdWithInfo(request);
    }

    /**
     * 根据用户id查询关联的急修
     *
     * @param request RepairRequest
     * @return IPage<RepairResponse>
     */
    public IPage<RepairResponse> pageByUser(RepairRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<RepairResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByUser(page, request);
    }

    /**
     * 急修接单，修改急修为修理中，修改电梯的业务状态为急修中
     *
     * @param entity RepairRequest
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean taking(EmergencyRepair entity) {
        boolean er = updateById(entity);
        if (!er) {
            return false;
        }
        PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(entity.getMtCompanyId(), entity.getLiftId());
        if (null == relevance) {
            return false;
        }
        //电梯业务状态改成急修中
        relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.EMERGENCY_REPAIR.getCode());
        boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
        if (re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 根据维保公司和电梯id查找急修
     *
     * @param mtCompanyId 维保公司id
     * @param liftId 电梯id
     * @return EmergencyRepair
     */
    public EmergencyRepair findByMtCompanyAndLift(Long mtCompanyId, Long liftId) {
        LambdaQueryWrapper<EmergencyRepair> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(EmergencyRepair::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(EmergencyRepair::getLiftId, liftId);
        //只查询一条
        lambdaQueryWrapper.last("LIMIT 1");
        return getOne(lambdaQueryWrapper);
    }

    /**
     * 电梯管理中根据电梯，维保公司和项目查询急修记录
     *
     * @param request RepairRequest
     * @return IPage<RepairResponse>
     */
    public IPage<RepairResponse> pageByCompanyLiftProject(RepairRequest request) {
        long pageSize = request.getPageSize();
        long pageNum = request.getPageNum();
        IPage<RepairResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByLift(page, request);
    }

    /**
     * 统计维保公司和电梯下是否存在待急修和急修中的急修单
     *
     * @param mtCompanyId 维保公司id
     * @param liftId 电梯id
     * @return int
     */
    public int countRepairing(Long mtCompanyId, Long liftId) {
        LambdaQueryWrapper<EmergencyRepair> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(EmergencyRepair::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(EmergencyRepair::getLiftId, liftId);
        lambdaQueryWrapper.in(EmergencyRepair::getStatus, Arrays.asList(Judge.Repair.STATUS_TO_REPAIRED, Judge.Repair.STATUS_REPAIRING));
        return count(lambdaQueryWrapper);

    }

    /**
     * 开启维保，暂停急修，修改电梯业务状态为维保中
     *
     * @param mtCompanyId 维保公司id
     * @param liftId 电梯id
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean startMaintenance(Long mtCompanyId, Long liftId) {
        // 如果急修单不存在，直接返回，
        EmergencyRepair emergencyRepair = findByMtCompanyAndLift(mtCompanyId, liftId);
        if (null == emergencyRepair) {
            return true;
        }
        //如果存在，不在急修中
        if (!Judge.Repair.statusRepairing(emergencyRepair.getStatus())) {
            return true;
        }
        // 如果存在，并且急修中,急修改成暂停中
        emergencyRepair.setStatus(Judge.Repair.STATUS_STOPING);
        boolean er = updateById(emergencyRepair);
        if (!er) {
            return false;
        }
        PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId, liftId);
        if (null == relevance) {
            return false;
        }
        //电梯业务状态改成维保中
        relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.MAINTENANCE.getCode());
        boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
        if (re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 停止维保，恢复急修，修改电梯业务状态为急修中
     *
     * @param mtCompanyId 维保公司id
     * @param liftId 电梯id
     * @return boolean
     */
    public boolean stopMaintenance(Long mtCompanyId, Long liftId) {
        //如果急修单不存在，直接返回，
        EmergencyRepair emergencyRepair = findByMtCompanyAndLift(mtCompanyId, liftId);
        if (null == emergencyRepair) {
            return true;
        }
        //如果急修单不在暂停中，直接返回true
        if (!Judge.Repair.statusStoping(emergencyRepair.getStatus())) {
            return true;
        }
        //恢复暂停中的急修为急修中
        emergencyRepair.setStatus(Judge.Repair.STATUS_REPAIRING);
        boolean er = updateById(emergencyRepair);
        if (!er) {
            return false;
        }
        PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId, liftId);
        if (null == relevance) {
            return false;
        }
        //电梯业务状态改成急修中
        relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.EMERGENCY_REPAIR.getCode());
        boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
        if (re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 生成急修单，同时插入急修图片记录
     *
     * @param repair EmergencyRepair
     * @param erRecordImgs List<ErRecordImg>
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean repairOrder(EmergencyRepair repair, List<ErRecordImg> erRecordImgs) {
        Judge.notNull(repair);
        Judge.notNull(erRecordImgs);
        boolean re = saveOrUpdate(repair);
        if (!re) {
            return false;
        }
        boolean im = erRecordImgService.saveBatch(erRecordImgs);
        if (im) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 插入急修收费项，同时更新急修记录总金额
     *
     * @param repair
     * @param erRecordCosts
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean addCost(EmergencyRepair repair, List<ErRecordCost> erRecordCosts) {
        boolean re = saveOrUpdate(repair);
        if (!re) {
            return false;
        }
        boolean er = erRecordCostService.saveBatch(erRecordCosts);
        if (er) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 修改收费项，更新急修总金额
     *
     * @param repair EmergencyRepair
     * @param erRecordCosts List<ErRecordCost>
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean modifyCost(EmergencyRepair repair, List<ErRecordCost> erRecordCosts) {
        boolean re = saveOrUpdate(repair);
        if (!re) {
            return false;
        }
        //先删除原有的收费项，再添加新的
//        List<ErRecordCost> costs = erRecordCostService.listByErRecordId(repair.getId());
//        if(IterUtil.isNotEmpty(costs)){
//            erRecordCostService.removeByIds(costs.stream().map(ErRecordCost::getId).collect(Collectors.toList()));
//        }
        //原有的更新，新加的新增
        boolean er = erRecordCostService.saveOrUpdateBatch(erRecordCosts,erRecordCosts.size());
        if (er) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 删除收费项，修改急修中的总金额
     *
     * @param repair EmergencyRepair
     * @param erRecordCost ErRecordCost
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteCost(EmergencyRepair repair, ErRecordCost erRecordCost) {
        boolean er = erRecordCostService.removeById(erRecordCost.getId());
        if (!er) {
            return false;
        }
        boolean re = updateById(repair);
        if (re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 急修评价，同时更新急修的评价状态
     *
     * @param repair EmergencyRepair
     * @param entity Evaluation
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean evaluate(EmergencyRepair repair, Evaluation entity) {
        boolean ev = evaluationService.save(entity);
        if (!ev) {
            return false;
        }
        boolean re = updateById(repair);
        if (re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }
}
