package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.MtRecordCost;
import cn.com.ty.lift.business.maintenance.dao.entity.MtRecordImg;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceAddRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenanceRecordResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtRecordResponse;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenanceRecordMapper;
import cn.com.ty.lift.business.maintenance.service.impl.MtRecordCostService;
import cn.com.ty.lift.business.maintenance.service.impl.MtRecordImgService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.utils.MapHelper;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author bieao
 * @date 2019/12/12
 * @description 保养记录业务层
 */
@Service
@AllArgsConstructor
public class MaintenanceRecordService extends ServiceImpl<MaintenanceRecordMapper, MaintenanceRecord> {


    private LiftService            liftService;
    private MtRecordCostService    mtRecordCostService;
    private MtRecordImgService     mtRecordImgService;
    private MaintenancePlanService maintenancePlanService;
    private EvaluationService      evaluationService;

    /**
     * @param request 公司id和电梯id
     * @return RestResponse 保养信息
     * @description 根据公司id和电梯id查询上次保养信息
     * @date 2020/1/6 5:37 下午
     */
    public MaintenanceRecordResponse recordInfo(MaintenanceRecordRequest request) {
        return baseMapper.findRecordByCompanyId(request);
    }

    /**
     * @param id 电梯id
     * @return 电梯经纬度
     * @description 根据电梯id获取经纬度
     * @date 2020/1/13 11:37 上午
     */
    public String getCoordinate(Long id){
        Lift lift = liftService.getById(id);
        return Optional.ofNullable(lift).map(Lift::getCoordinate).orElse(null);
    }


    /**
     * @param coordinate 维保工经纬度
     * @return 是否在范围内
     * @description 计算维保工开始保养时，是否在电梯200米范围内
     * @date 2020/1/13 11:30 上午
     */
    public boolean calc(Long id, String coordinate) {
        String liftCoordinate = getCoordinate(id);
        double expected = 200;
        double result = MapHelper.GetPointDistance(liftCoordinate, coordinate);
        return (result * 1000) > expected;
    }

    /**
     * @param planId 计划id
     * @param mtCompanyId 公司id
     * @return 是否存在
     * @description 根据计划id和公司id查询是否有保养记录
     * @date 2020/1/20 2:41 下午
     */
    public boolean count(Long planId, Long mtCompanyId) {
        LambdaQueryWrapper<MaintenanceRecord> lambdaQueryWrapper = new QueryWrapper<MaintenanceRecord>().lambda();
        lambdaQueryWrapper.eq(MaintenanceRecord::getMtPlanId, planId);
        lambdaQueryWrapper.eq(MaintenanceRecord::getMtCompanyId, mtCompanyId);
        return count(lambdaQueryWrapper) > 0;
    }

    /**
     * @param request 保养单信息
     * @return 是否成功
     * @description 保存保养记录
     * @date 2020/1/13 1:47 下午
     */
    public RestResponse saveRecord(MaintenanceAddRecordRequest request) {
        MaintenanceRecord record = request.getRecord();
        Long planId = record.getMtPlanId();
        Long mtCompanyId = record.getMtCompanyId();
        boolean count = count(planId,mtCompanyId);
        if (count) return  RestResponse.fail(MessageUtils.get("msg.exist.record"));
        boolean result = calc(record.getLiftId(), request.getCoordinate());
        if (result) return RestResponse.fail(MessageUtils.get("msg.error.person.scope"));
        Long liftId = record.getLiftId();
        Optional<Lift> lift = liftService.getOne(null, liftId);
        if (lift.isPresent()) {
            Lift l = lift.get();
            record.setCode(l.getLiftCode());
            record.setLiftType(l.getLiftType());
            record.setStatus(CommonEnum.MaintenanceRecordStatus.SIGN.getCode());
            record.setWorkDate(LocalDateTime.now());
            record.setStopDate(LocalDateTime.now());
            MaintenancePlan plan = maintenancePlanService.getOne(record.getMtPlanId());
            if (ObjectUtil.isEmpty(plan)) return RestResponse.fail(MessageUtils.get("msg.add.fail"));
            record.setPlanDate(plan.getPlanDate());
            boolean ret = save(record);
            if (!ret) {
                return RestResponse.fail(MessageUtils.get("msg.add.fail"));
            }
        } else return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        return RestResponse.success(record.getId(), MessageUtils.get("msg.add.success"));
    }

    /**
     * @param record 保养信息
     * @return 是否成功
     * @description 编辑保养信息
     * @date 2020/1/13 2:31 下午
     */
    public boolean modify(MaintenanceRecord record){
        record.setRecoveryDate(LocalDateTime.now());
        record.setStatus(CommonEnum.MaintenanceRecordStatus.EVALUATE.getCode());
        return updateById(record);
    }

    /**
     * @param id 保养记录id
     * @return 保养信息详情
     * @description 查看保养信息详情
     * @date 2020/1/13 2:39 下午
     */
    public MaintenanceRecord detail(Long id) {
        return getById(id);
    }

    /**
     * 根据条件分页查询维保-日常保养
     * @author wcz
     * @param request MtRecordRequest
     * @return IPage<MtRecordResponse>
     */
    public IPage<MtRecordResponse> pageRecordByCondition(MtRecordRequest request){
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<MtRecordResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageRecordByCondition(page,request);
    }

    /**
     * 根据维保公司统计维保-日常保养
     * @author wcz
     * @param request MtRecordRequest
     * @return long
     */
    public long countRecordByCondition(MtRecordRequest request){
        return baseMapper.countRecordByCondition(request);
    }

    /**
     * 日常保养-保养单：根据id查询,带关联信息
     * @author wcz
     * @param request MtRecordRequest
     * @return boolean
     */
    public MtRecordResponse findByIdWithInfo(MtRecordRequest request){
        return baseMapper.findByIdWithInfo(request);
    }

    /**
     * 点击填写保养单，带收费项，新增record
     * @author wcz
     * @param entity MaintenanceRecord
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean tofill(MaintenanceRecord entity){
        //总计 partsCost, manCost
        List<MtRecordCost> mtRecordCosts = entity.getMtRecordCosts();
        if(IterUtil.isNotEmpty(mtRecordCosts)){
            BigDecimal partsCost = mtRecordCosts.stream().map(MtRecordCost::getPartsCost).reduce(BigDecimal.ZERO, (m1, m2) -> m1.add(m2));
            BigDecimal manCost = mtRecordCosts.stream().map(MtRecordCost::getManCost).reduce(BigDecimal.ZERO, (m1, m2) -> m1.add(m2));
            entity.setPartsCost(partsCost);
            entity.setManCost(manCost);
        }
        boolean mr = saveOrUpdate(entity);
        if(!mr){
            return false;
        }
        //如果没有收费项
        if(IterUtil.isEmpty(mtRecordCosts)){
            return true;
        }
        mtRecordCosts.forEach(cost ->{
            cost.setMtRecordId(entity.getId());
        });
        boolean rc = mtRecordCostService.saveOrUpdateBatch(mtRecordCosts, mtRecordCosts.size());
        if(rc){
            return true;
        }else{
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 填写保养单-确认，带维保项和现场图片 ，修改record
     * @author wcz
     * @param entity MaintenanceRecord
     * @param imgs List<String>
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean fill(MaintenanceRecord entity,List<String> imgs){
        boolean mr = saveOrUpdate(entity);
        if(!mr){
            return false;
        }
        if(IterUtil.isEmpty(imgs)){
            return true;
        }
        List<MtRecordImg> mtRecordImgs = new ArrayList<>();
        for (int i = 0; i < imgs.size(); i++) {
            MtRecordImg mri = new MtRecordImg();
            mri.setMtRecordId(entity.getId());
            mri.setImgUrl(imgs.get(i));
            mri.setSort(i);
            mtRecordImgs.add(mri);
        }
        boolean mi = mtRecordImgService.saveBatch(mtRecordImgs,mtRecordImgs.size());
        if(mi){
            return true;
        }else{
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 通过：app端提交的保养单点通过后生成正式保养单，清除超期； -> status:-1修改成0
     * @param entity MaintenanceRecord
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean pass(MaintenanceRecord entity){
        boolean mr = updateById(entity);
        if(!mr){
            return false;
        }
        boolean mp = maintenancePlanService.removeById(entity.getMtPlanId());
        if(mp){
            return true;
        }else{
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }

    /**
     * 维保记录评价，同时更新维保记录状态 status -> 3
     * @param record
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean evaluate(MaintenanceRecord record, Evaluation entity){
        boolean ev = evaluationService.save(entity);
        if(!ev){
            return false;
        }
        boolean mr = updateById(record);
        if(mr){
            return true;
        }else{
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }
}
