package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.project.dao.entity.ProjectUser;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectUserRequest;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectUserResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectUserMapper;
import cn.com.ty.lift.common.constants.CommonEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2019/12/31
 * @description 项目组成员业务层
 */
@Service
public class ProjectUserService extends ServiceImpl<ProjectUserMapper, ProjectUser> {

    @Resource
    private ProjectUserMapper projectUserMapper;

    public List<ProjectUserResponse> getUserList(Long projectId) {
        return projectUserMapper.findUserListById(projectId);
    }

    /**
     * @param projectId 项目ID
     * @return List<ProjectUser> userList项目组成员列表
     * @description 查询项目组成员
     * @date 2019/12/13 2:40 PM
     */
    public List<ProjectUserResponse> userList(Long projectId) {
        List<ProjectUserResponse> userList = getUserList(projectId);
        return userList.stream().filter(user -> (
                CommonEnum.DefaultRole.MTWORK.getCode().equals(user.getUserRole())
                        || CommonEnum.DefaultRole.REGION_DIRECTOR.getCode().equals(user.getUserRole())
        )).sorted(Comparator.comparing(ProjectUserResponse::getUserRole)).collect(Collectors.toList());
    }

    /**
     * @param request 批量新增项目组成员
     * @return Integer 成功1或失败0
     * @description 批量新增项目组成员
     * @date 2019/12/4 4:41 PM
     */
    public boolean add(ProjectUserRequest request) {
        List<ProjectUser> userList = request.getUserList();
        userList.forEach(entry -> {
            entry.setProjectId(request.getProjectId());
            entry.setMtCompanyId(request.getMtCompanyId());
            entry.setUserRole(CommonEnum.DefaultRole.MTWORK.getCode());
        });
        return saveBatch(userList);
    }

    /**
     * @param projectUser 修改项目组成员
     * @return Integer 成功1或失败0
     * @description 修改项目组成员
     * @date 2020/1/11 10:43 上午
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean modify(ProjectUser projectUser){
        QueryWrapper<ProjectUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", projectUser.getUserId());
        queryWrapper.eq("project_id", projectUser.getProjectId());
        queryWrapper.eq("user_role", CommonEnum.DefaultRole.MTWORK.getCode());
        ProjectUser entry = getOne(queryWrapper);
        //如果该成员是维保工，升级成维保班长，若该项目组成员还有其他维保班长，则该维保班长设置为维保工
        if ("0".equals(entry.getIsMonitor())) {
            //设置为维保班长
            entry.setIsMonitor("1");
            boolean result = update(entry, queryWrapper);
            if (!result) return false;
            QueryWrapper<ProjectUser> wrapper = new QueryWrapper<>();
            wrapper.eq("mt_company_id", projectUser.getMtCompanyId());
            wrapper.eq("project_id", projectUser.getProjectId());
            wrapper.eq("user_role", CommonEnum.DefaultRole.MTWORK.getCode());
            List<ProjectUser> userList = list(wrapper);
            Optional<ProjectUser> first = userList.stream().filter(user ->
                    !user.getUserId().equals(entry.getUserId()) && user.getIsMonitor().equals("1")).findFirst();
            if (first.isPresent()) {
                ProjectUser user = first.get();
                //设置另外的维保班长为维保工
                user.setIsMonitor("0");
                boolean ret = update(user, new QueryWrapper<ProjectUser>() {{
                    eq("user_id", user.getUserId());
                    eq("project_id", user.getProjectId());
                    eq("user_role", CommonEnum.DefaultRole.MTWORK.getCode());
                }});
                if (ret) return true;
                else TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            }
            return true;
        } else {
            //设置为维保工
            entry.setIsMonitor("0");
            return update(entry, queryWrapper);
        }
    }

    /**
     * @param request 批量删除项目组成员
     * @return Integer 成功1或失败0
     * @description 批量删除项目组成员
     * @date 2020/1/11 10:43 上午
     */
    public int remove(ProjectUserRequest request) {
        List<ProjectUser> userList = request.getUserList();
        userList.forEach(entry -> {
            entry.setProjectId(request.getProjectId());
            entry.setMtCompanyId(request.getMtCompanyId());
            entry.setUserRole(CommonEnum.DefaultRole.MTWORK.getCode());
        });
        return projectUserMapper.batchRemoveWithParams(request);
    }
}
