package cn.com.ty.lift.enterprise.common;

import cn.com.ty.lift.common.utils.Judge;
import cn.com.ty.lift.common.utils.JudgeException;
import cn.com.ty.lift.common.utils.Judger;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.time.Duration;
import java.time.Instant;
import java.util.Arrays;

/**
 * @author bieao
 * @date 2019/12/3 10:30 AM
 * @description 日志切面
 */
@Slf4j
@Aspect
@Component
public class ControllerAspect {

    private static final String  line        = "============================================================";
    private static final String  head        = "#####| ";
    private static final String  url         = head + "URL          : ";
    private static final String  httpMethod  = head + "HTTP_METHOD  : ";
    private static final String  ip          = head + "IP           : ";
    private static final String  classMethod = head + "CLASS_METHOD : ";
    private static final String  args        = head + "ARGS         : ";
    private static final String  resp        = head + "RESPONSE     : ";
    private static final String  proceed     = head + "TIME         : ";
    private static final boolean printAll    = false;
    private static final int     printLength = 1000;
    private static final long    goodtime    = 10 * 1000;

    @Pointcut("execution(* cn.com.ty.lift.enterprise.*.controller..*(..))")
    public void pointcut() {
    }

    @Before("pointcut()")
    public void doBefore(JoinPoint joinPoint) throws Throwable {
        // Receives the request and get request content
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        log.info(line);
        log.info(url + request.getRequestURL().toString());
        log.info(httpMethod + request.getMethod());
        log.info(ip + request.getRemoteAddr());
        log.info(classMethod + joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName());
        log.info(args + Arrays.toString(joinPoint.getArgs()));

        judge(joinPoint);
    }

    @AfterReturning(returning = "response", pointcut = "pointcut()")
    public void doAfterReturning(RestResponse response) throws Throwable {
        // Processes the request and returns the content
        String res = JSONUtil.toJsonPrettyStr(response);
        if(!printAll && null != res){
            int length = res.length();
            res = length > printLength ? res.substring(0, printLength) + "\n... "+ (length - printLength) +" characters omitted." : res;
        }
        log.info(resp + "\n" + res);
        log.info(line);
    }

    @AfterThrowing(throwing = "ex", pointcut = "pointcut()")
    public void doAfterThrowing(Exception ex) throws Throwable {
        // Processes the exception
        if(null != ex){
            Class<? extends Exception> exClass = ex.getClass();
            if(exClass.equals(JudgeException.class) || exClass.equals(MethodArgumentNotValidException.class)){
                final String msg = ex.getMessage();
                log.info(resp + msg);
                log.info(line);
            }
        }
    }


    @Around("pointcut()")
    public Object interceptor(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        Instant start = Instant.now();
        Object result = proceedingJoinPoint.proceed();
        Instant end = Instant.now();
        long toMillis = Duration.between(start, end).toMillis();
        if (goodtime < toMillis) {
            Signature signature = proceedingJoinPoint.getSignature();
            String classMethod = signature.getDeclaringTypeName() + "." + signature.getName();
            log.warn(proceed + " (" + toMillis + " ms)" + classMethod);
        } else {
            log.info(proceed + toMillis + " ms");
        }
        return result;
    }

    //检查必要参数非空,数值型参数必须 !null && > 0
    private void judge(JoinPoint joinPoint){
        Object[] args = joinPoint.getArgs();
        MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
        Method method = methodSignature.getMethod();
        Judger judger = method.getDeclaredAnnotation(Judger.class);
        if(null == judger || args.length == 0){
            return;
        }
        Object object = args[0];
        String[] fields = judger.fields();
        if(null == object || fields.length == 0){
            return;
        }
        Judge.judgement(object,fields);
    }
}
