package cn.com.ty.lift.enterprise.common;

import cn.com.ty.lift.common.utils.JudgeException;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author wcz
 * @date 2019/12/10
 * @description 全局异常捕获处理
 */
@Slf4j
@RestControllerAdvice
public class GlobalDefaultExceptionHandler {

    private static final String classPrefix = "cn.com.ty.lift.enterprise";
    private static final String nameSuffix  = ".java";
    private static final String line        = "========================= GlobalDefaultException =========================";
    private static final String head        = "#####| ";
    private static final String url         = head + "URL          : {}";
    private static final String status      = head + "STATUS       : {}";
    private static final String exception   = head + "EXCEPTION    : ";

    //处理judge抛出的异常
    @ExceptionHandler(value = JudgeException.class)
    public Object judgeHandler(final JudgeException e) {
        // 捕获错误message
        String message = e.getMessage();
        return StrUtil.isEmpty(message) ? RestResponse.failParam() : RestResponse.fail(message);
    }

    //全局异常捕获处理
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public Object validHandler(final MethodArgumentNotValidException e) {
        // 获取验证框架的错误详情
        final FieldError error = e.getBindingResult().getFieldError();
        // 捕获错误message
        String message = error.getDefaultMessage();
        if (message.equals("notEmpty") || message.startsWith("msg") || message.startsWith("err")) {
            return RestResponse.failParam();
        } else {
            return StrUtil.isEmpty(message) ? RestResponse.failParam() : RestResponse.fail(message);
        }
    }

    @ExceptionHandler(value = ExceptionInInitializerError.class)
    @ResponseStatus(value = HttpStatus.BAD_REQUEST)
    public Object badRequest() {
        return RestResponse.exception("Bad Request");
    }

    @ExceptionHandler(value = Exception.class)
    public Object defaultErrorHandler(final HttpServletRequest req, final HttpServletResponse resp, final Exception error) {
        // 原始错误集合数据偏多，包含native错误
        final StackTraceElement[] stackTrace = error.getStackTrace();

        // 过滤错误信息
        final List<StackTraceElement> elements = Stream.of(stackTrace)
                .filter(st -> (st.getClassName().contains(classPrefix) && st.getFileName().contains(nameSuffix)))
                .collect(Collectors.toCollection(LinkedList::new));
        final StackTraceElement[] dwStackTrace = new StackTraceElement[elements.size()];
        elements.toArray(dwStackTrace);

        // 填充StackTrace
        error.setStackTrace(dwStackTrace);
        log.error(line);
        log.error(url, req.getRequestURI());
        log.error(status, resp.getStatus());
        log.error(exception, error);

        if (error instanceof EnterpriseBasicException) {
            final EnterpriseBasicException enterpriseBasicException = (EnterpriseBasicException) error;
            return RestResponse.fail(enterpriseBasicException.addSuffix(error.getMessage()).replace("()", ""));
        }
        return RestResponse.exception();
    }
}
