package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.aliservice.constants.AliConstants;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.system.settings.dao.entity.MaintenanceCompany;
import cn.com.ty.lift.system.settings.service.IMaintenanceCompanyService;
import cn.com.ty.lift.system.user.dao.entity.Menu;
import cn.com.ty.lift.system.user.dao.entity.Role;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.UserRole;
import cn.com.ty.lift.system.user.dao.entity.model.UserRequest;
import cn.com.ty.lift.system.user.dao.entity.model.UserResponse;
import cn.com.ty.lift.system.user.service.*;
import cn.com.ty.lift.system.utils.PasswordUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author huangyuan
 * @date 2019-11-29
 * @description 用户登录接口
 */
@Service
public class LoginService implements ILoginService {

    @Resource
    private IUserService userService;

    @Resource
    private IUserAccountService userAccountService;

    @Resource
    private RedisTemplate redisTemplate;

    @Resource
    private IUserLoginService userLoginService;

    @Resource
    private IRoleService roleService;

    @Resource
    private IUserRoleService userRoleService;

    @Resource
    private IRoleMenuService roleMenuService;

    @Resource
    private IMaintenanceCompanyService maintenanceCompanyService;

    @Resource
    private IMenuService menuService;

    @Override
    public RestResponse register(HttpServletRequest request, UserRequest userRequest) {
        //判断手机号是否为空
        if (StringUtils.isBlank(userRequest.getMobile())) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号为空");
        }
        //判断密码是否为空
        if (StringUtils.isBlank(userRequest.getPassword())) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "密码为空");
        }
        //校验短信验证码
        RestResponse verifyResult = verifySmsCode(userRequest.getMobile(), userRequest.getInputSmsCode());
        //如果结果不为空说明校验未通过
        if (verifyResult != null) {
            return verifyResult;
        }
        //判断手机号是否注册过
        UserAccount userAccount = userAccountService.getByMobile(userRequest.getMobile());
        if (userAccount != null) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "用户手机号已经注册过了");
        }

        //保存用户信息,并返回用户账户信息
        userAccount = userService.saveUserInfo(userRequest);
        if (userAccount != null) {
            return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "注册成功");
        }
        return RestResponse.fail(ApiConstants.RESULT_ERROR, "注册失败");
    }

    @Override
    public RestResponse login(HttpServletRequest request, UserRequest userRequest) {
        String password = userRequest.getPassword();
        if (StringUtils.isBlank(userRequest.getMobile())) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号为空无法登录");
        }
        if (StringUtils.isBlank(password)) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "用户密码为空无法登录");
        }
        //通过手机号获取用户账户信息
        UserAccount userAccount = userAccountService.getByMobile(userRequest.getMobile());
        //验证手机号是否存在
        if (userAccount == null) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号尚未注册");
        }
        //校验密码是否正确
        String inputPassword = PasswordUtils.generatePassword(password, userAccount.getSalt());
        if (!inputPassword.equals(userAccount.getPassword())) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "密码输入有误");
        }
        //登录成功后初始化登录信息
        UserResponse userResponse = initLoginInfo(request, userAccount.getUserId());
        //添加登录记录
        userLoginService.saveLoginInfo(request, userAccount.getUserId());
        return RestResponse.success(userResponse, ApiConstants.RESULT_SUCCESS, "登录成功");
    }

    @Override
    public RestResponse resetPassword(UserRequest userRequest) {
        RestResponse verifyResult = verifySmsCode(userRequest.getMobile(), userRequest.getInputSmsCode());
        //如果结果不为空说明校验未通过
        if (verifyResult != null) {
            return verifyResult;
        }
        //获取用户信息
        UserAccount userAccount = userAccountService.getByMobile(userRequest.getMobile());
        if (userAccount == null) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号尚未注册");
        }
        String salt = userAccount.getSalt();
        //新密码加密
        String newPassword = PasswordUtils.generatePassword(userRequest.getPassword(), salt);
        userAccount.setPassword(newPassword);
        userAccountService.updateById(userAccount);
        //用户修改密码后，需要重新登录，将redis中的用户信息置空
        String mobileUserInfo = (String) redisTemplate.opsForValue().get(userRequest.getToken());
        redisTemplate.opsForValue().set(userRequest.getToken(), null);
        redisTemplate.opsForValue().set(mobileUserInfo, null);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "重置密码成功");
    }

    @Override
    public RestResponse changeTeam(HttpServletRequest request, UserRequest userRequest) {
        Long companyId = userRequest.getCompanyId();
        UserResponse userResponse = new UserResponse();
        //校验团队信息
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(companyId);
        if (maintenanceCompany == null) {
            return RestResponse.success(ApiConstants.RESULT_SUCCESS, "切换团队不存在");
        }
        maintenanceCompany.setCurrentTeamFlag(true);
        userResponse.setMaintenanceCompany(maintenanceCompany);
        UserRole userRole = userRoleService.getOne(new QueryWrapper<UserRole>()
                .eq("company_id", companyId)
                .eq("user_id", userRequest.getUserId())
        );
        if (userRole != null) {
            Role role = roleService.getById(userRole.getRoleId());
            userResponse.setRole(role);
            //获取菜单信息
            if (role != null) {
                //设置菜单树
                List<Menu> menuTree = roleMenuService.getMenuTree(role.getId());
                userResponse.setMenus(menuTree);
            }
        }
        //更新redis中数据值
        return RestResponse.success(userResponse, ApiConstants.RESULT_SUCCESS, "切换团队成功");
    }

    @Override
    public RestResponse wxLogin(HttpServletRequest request, UserRequest userRequest) {
        //通过授权码获取用户openId
        String openId = userRequest.getAuthCode();
        UserAccount userAccount = userAccountService.getOne(new QueryWrapper<UserAccount>()
                .eq("wechat_open_id", openId)
        );
        if (userAccount == null) {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "微信号尚未绑定用户信息");
        }
        //用户存在，关联登录成功
        //登录成功初始化登录信息
        UserResponse userResponse = initLoginInfo(request, userAccount.getUserId());
        //保存登录记录
        userLoginService.saveLoginInfo(request, userAccount.getUserId());
        return RestResponse.success(userResponse, ApiConstants.RESULT_SUCCESS, "用户登录成功");
    }

    /**
     * @param
     * @return
     * @description 初始化登录信息
     * @date 2019-12-30 11:22
     */
    private UserResponse initLoginInfo(HttpServletRequest request, Long userId) {
        //获取用户信息
        UserResponse userResponse = userService.getLoginUserInfo(userId);
        //将用户手机号放入session中
        request.getSession().setAttribute(ApiConstants.CURRENT_USER_ID, userResponse.getUserId());
        //封装登录用户信息
        Map<String, Object> currentUserInfoMap = getUserInfoMap(userResponse);
        String mobileUserInfo = userResponse.getMobile()
                + ApiConstants.UserConstants.USER_INFO_MOBILE;
        //将token对应用户手机号放入redis中,设置过期时间为30分钟
        redisTemplate.opsForValue().set(userResponse.getToken(), mobileUserInfo, 30L, TimeUnit.MINUTES);
        //将用户手机号对应的用户信息放入redis中, 过期时间为30分钟
        redisTemplate.opsForValue().set(mobileUserInfo, JSONUtil.toJsonPrettyStr(currentUserInfoMap),
                30L, TimeUnit.MINUTES);
        return userResponse;
    }

    @Override
    public RestResponse verifySmsCode(String mobile, String inputSmsCode) {
        Object smsCode = redisTemplate.opsForValue().get(mobile + AliConstants.SmsConstants.SMS_CODE_FIELD);
        if (smsCode != null) {
            if (!(smsCode.equals(inputSmsCode))) {
                return RestResponse.fail(ApiConstants.RESULT_ERROR, "短信验证码输入有误");
            }
        } else {
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "验证码过期，请重新获取验证码");
        }
        return null;
    }

    /**
     * @param userResponse 用户信息
     * @return map 用户信息map
     * @description 封装放入redis的用户信息
     * @date 2020-01-16 11:40
     */
    private Map<String, Object> getUserInfoMap(UserResponse userResponse) {
        //封装用户登录信息，并将登录信息放入redis中
        Map<String, Object> currentUserInfoMap = new HashMap<>();
        //当前用户id
        currentUserInfoMap.put(ApiConstants.CURRENT_USER_ID, userResponse.getUserId());
        //如果用户类型为空，设置用户类型为普通类型
        int userType = 3;
        if (userResponse.getType() != null) {
            userType = userResponse.getType();
        }
        //当前用户信息
        currentUserInfoMap.put(ApiConstants.CURRENT_USER_TYPE, userType);
        //当前公司id
        MaintenanceCompany maintenanceCompany = userResponse.getMaintenanceCompany();
        if (maintenanceCompany != null) {
            currentUserInfoMap.put(ApiConstants.CURRENT_COMPANY_ID, maintenanceCompany.getId());
        }
        //当前角色id
        Role role = userResponse.getRole();
        if (role != null) {
            currentUserInfoMap.put(ApiConstants.CURRENT_ROLE_ID, role.getId());
        }
        //用户权限信息
        String permissionUrl = menuService.menuToUrl(userResponse.getMenus());
        currentUserInfoMap.put(ApiConstants.CURRENT_PERMISSION_URL, permissionUrl);
        return currentUserInfoMap;
    }
}
